<?php if (!defined('PmWiki')) exit();
$RecipeInfo['SimpleChem']['0.9a'] = '2008-11-08';

/* $Id: SimpleChem.php,v 1.3 2008/11/08 18:59:54 schilkek Exp schilkek $ */

/** \file SimpleChem.php
 * \brief Simple inline markup to make writing chemical formulas easier.
 *
 * +----------------------------------------------------------------------+
 * | See cookbook/simplechem/README.txt for information.
 * | See cookbook/simplechem/LICENSE.txt for licence.
 * +----------------------------------------------------------------------+
 * | Copyright 2008 Karl "Rat" Schilke (schilkek@onid.orst.edu)
 * | This program is free software; you can redistribute it and/or modify
 * | it under the terms of the GNU General Public License, Version 2, as
 * | published by the Free Software Foundation.
 * | http://www.gnu.org/copyleft/gpl.html
 * | This program is distributed in the hope that it will be useful,
 * | but WITHOUT ANY WARRANTY; without even the implied warranty of
 * | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * | GNU General Public License for more details.
 * +----------------------------------------------------------------------+
 *
 * We want to subscript numbers in chemical formulae like '-NH2', 'H2O',
 * 'C2H5OH', or '(CH3)2N(C6H4)CH2N=NCH2(C6H4)SO3Cl'.
 * 
 * A simple, transparent rule (simplechem) is sufficient for formulas like 
 * 'R-NH2', 'H2O', 'CO2', etc.  Text that is incorrectly marked-up can be
 * wrapped in '[= ... =]' tags to prevent subscripting.
 * 
 * For more complicated formulas, the start of the formula is signalled with a
 * leading '&' character. This allows more advanced formula conversions. There
 * is no collision with special characters, but the formula tag could be changed
 * to an explicit tag, e.g. '(:chem ... :)'.
 * 
 * Note that this needs to be done after links are resolved, to avoid messing 
 * up links like 'This [[CHEMSKETCH2 | link to CHEMSKETCH2]] doesn't work
 * because the 2 in H2 is subscripted.'
 *
 * This recipe is based on suggestions by Peter Bowers (pbowers_at_pobox_com)
 * and John Rankin (john_rankin_at_affinity_co_nz).
 */ 

Markup('simplechem', '>links', '/((?:Cl|Br|Si|Na|[CHNOPS])+\)?)(\d+)/',
	'$1<sub>$2</sub>');


/* More advanced chemistry markup: &formula is interpreted as follows:
 *
 * Subscript strings of numbers in the chemical formula, replace '.' with an
 * HTML middot character, convert '==' to a triple bond, and trailing '*'
 * becomes a radical. Trailing '+' or '-' characters are counted and replaced
 * with a charge (e.g. SO4-- becomes "SO<SUB>4</SUB><SUPER>2-</SUPER>").
 * Note that subscripting should not happen for the number following a '.'
 * (as in FeCl3.6H2O - the '6' should not be subscripted). "R-groups" should
 * be superscripted, not subscripted, to distinguish them from other groups.
 *
 * Note: It is up to the author to ensure that the formula makes sense from
 * a physicochemical standpoint... no carbon atoms with 5 substituents, please!
 */

Markup('advchem', '>links',
	'/\&amp;([A-Z\(][A-Za-z()=\-\.\d\*]*[A-Za-z\d\*])(\++|-+)?/e',
	    "FixupChemFormula('$1') . ('$2' ? SuperscriptCharge('$2') : '')");

/*
 * FixupChemFormula() marks up various parts of the chemical formula string,
 * as described above. 
 */

function FixupChemFormula($formula) {
    $formula = preg_replace('/(R)(\d+)/',        "$1<SUP>$2</SUP>", $formula);
    $formula = preg_replace('/([^R\.])(\d+)/',   "$1<SUB>$2</SUB>", $formula);
    $formula = preg_replace('/\)([a-z])/',       ")<SUB><I>$1</I></SUB>",
                                                                    $formula);
    $formula = preg_replace('/\./',              "&middot;",        $formula);
    $formula = preg_replace('/==/',              "&equiv;",         $formula);
    $formula = preg_replace('/\*/',              "&bull;",          $formula);
    return "$formula";
}

/*
 * SuperscriptCharge() counts the number of '+' or '-' characters trailing
 * the formula, and writes a charge in the form '2+' or '3-'. If the charge
 * is (+/-)1, the number is not displayed. Because superscripted '-' is very
 * small, negative charges are rendered as longer 'n-dash' characters.
 */

function SuperscriptCharge($charge) {
    $num  = strlen($charge); /* Count number of +/- characters in the charge */

    /* Determine the "sign" ("+" or an n-dash) from the first +/- char. */
    $sign = ($charge[0] == '+' ? '+' : '&ndash;');

    return '<SUP>' . (($num > 1) ? "$num$sign" : $sign) . '</SUP>';
}


